;*********************************************************************
;	TEST690.ASM	MPB  Ver 1.0
;	Test program for DIZI690 demo board
; 	Status: Under development 1-1-11
;
;*********************************************************************

	PROCESSOR 16F690	; Specify MCU for assembler
	__CONFIG 00E5		; MCU configuration bits
				; PWRT on, MCLR enabled 
				; Internal Clock (default 4MHz)
	INCLUDE "P16F690.INC"	; Standard register labels

Loco	EQU	20		; Low count register
Hico	EQU	21		; High count register
Hexnum	EQU	22		; Hex table offset

; Initialise registers................................................

	BANKSEL	ANSEL		; Select Bank 2
	CLRF	ANSEL		; Ports digital I/O
	BSF	ANSEL,0		; except AN0 Analogue input
	CLRF	ANSELH		; Ports digital I/O

	BANKSEL	TRISC		; Select Bank 1
	CLRF	TRISC		; Port C for output
	MOVLW	B'00010000'	; A/D clock setup code
	MOVWF	ADCON1		; A/D clock = fosc/8

	BANKSEL PORTC		; Select bank 0
	CLRF	PORTC		; Clear display outputs
	MOVLW	B'00000001'	; Analogue input setup code
	MOVWF	ADCON0		; Left justify, Vref=5V, 
				; Select RA0, done, enable A/D 

; Main loop .........................................................

buta	BTFSC	PORTA,2		; Check button A
	GOTO	butb		; ..and button B if off
	CLRF	PORTC		; Clear display
	BSF	PORTC,0		; Switch on LSB

rotdis	RLF	PORTC		; Rotate segment
	CALL	vardel		; Run variable delay 
	BTFSS	PORTA,2		; Check button A again
	GOTO	rotdis		; ..and continue if on

butb	BTFSC	PORTA,5		; Else check button B
	GOTO	buta		; ..and repeat if off
	CALL	hexdis		; Get display code
	MOVWF	PORTC		; Show it
	GOTO	buta		; and repeat always

; Delay subroutine..................................................

vardel	BSF	ADCON0,1	; start ADC..
wait	BTFSC	ADCON0,1	; ..and wait for finish
	GOTO	wait
	MOVF	ADRESH,W	; Store result high byte
	MOVWF	Hico
	INCF	Hico		; Avoid zero count

slow	CLRF	Loco		; use result in delay
fast	DECFSZ	Loco
	GOTO	fast
	DECFSZ	Hico
	GOTO	slow
	RETURN			; done

; Hex display subroutine ...........................................

hexdis	MOVF	PORTB,W		; Get DIP switches
	ANDLW	B'11110000'	; Mask low bits
	MOVWF	Hexnum		
	SWAPF	Hexnum		; Swap hi-lo bits
	MOVF	Hexnum,W
	
	ADDWF	PCL		; Hex code table
	RETLW	B'01111110'	; 0
	RETLW	B'00001100'	; 1
	RETLW	B'10110110'	; 2
	RETLW	B'10011110'	; 3
	RETLW	B'11001100'	; 4
	RETLW	B'11011010'	; 5
	RETLW	B'11111010'	; 6
	RETLW	B'00001110'	; 7
	RETLW	B'11111110'	; 8
	RETLW	B'11001110'	; 9
	RETLW	B'11101110'	; A
	RETLW	B'11111000'	; b
	RETLW	B'01110010'	; C
	RETLW	B'10111100'	; d
	RETLW	B'11110010'	; E
	RETLW	B'11100010'	; F

	END	; Terminate assembler...............................

