;
;       Source File:	BIN4.ASM		
;	Author:		M. Bates              
;	Date:		15-10-03      
;       .....................................................
;       Program Description:
;
;	Slow output binary count is stopped, started
;       and reset with push buttons. This version uses a 
;	subroutine for the delay....
;
;       Processor:      PIC 16F84
;
;       Hardware:       PIC Demo System    
;       Clock:          CR ~100kHz
;       Inputs:         Push Buttons RA0, RA1 (active low)
;       Outputs:        LEDs (active high)
;       
;       WDTimer:        Disabled
;       PUTimer:        Enabled
;       Interrupts:     Disabled
;       Code Protect:   Disabled
;
; ***********************************************************

; Register Label Equates.....................................
        
porta   EQU     05             	; Port A Data Register  
portb   EQU     06             	; Port B Data Register
timer   EQU     0C             	; Spare register for delay

; Input Bit Label Equates ...................................

inres   EQU     0              	; 'Reset' input button = RA0
inrun   EQU     1              	; 'Run' input button = RA1

; ***********************************************************

; Initialise Port B (Port A defaults to inputs)..............
        
       	MOVLW   b'00000000'    	; Port B Data Direction Code
    	TRIS    portb          	; Load the DDR code into F86
	GOTO	reset


; 'delay' subroutine ........................................
        
delay  	MOVWF  	timer          	; Copy W to timer register
down    DECFSZ 	timer          	; Decrement timer register 
        GOTO   	down           	; and repeat until zero
	RETURN			; Jump back to main program


; Start main loop ...........................................

reset   CLRF    portb  		; Clear Port B Data 

start   BTFSS  	porta,inres    	; Test RA0 input button
       	GOTO   	reset          	; and reset Port B if pressed
        BTFSC  	porta,inrun    	; Test RA1 input button
        GOTO  	start          	; and run count if pressed

        INCF   	portb          	; Increment count at Port B 
	MOVLW  	0FF            	; Delay count literal
	CALL	delay		; Jump to subroutine 'delay'

        GOTO   	start          	; Repeat main loop always
        END                   	; Terminate source code
