; **************************************************************
;        TIM1.ASM       MPB 	02-02-11
; **************************************************************
;
;  Minimal program to demonstrate the hardware timer operation.
;
;  The counter/timer register (TMR0) is initialised to 
;  zero and diven from the instruction clock with a 
;  prescale value of 64.
;
;  T0IF is polled while the program waits for time out.
;  When the timer overflows, the Timer Interrupt Flag (T0IF) is 
;  set. The output LED binary display is then incremented.
;  With the clock adjusted to 65536 Hz, the LSB LED flashes at 
;  1 Hz.
;
;       Processor:      PIC 16F84A
;
;       Hardware:       Simulation version   
;       Clock:          CR = 65536 Hz (approx)
;       Outputs:        RB0 - RB7: LEDs (active high)
;       WDTimer:        Disabled
;       PUTimer:        Enabled
;       Interrupts:     Disabled
;       Timer:          Internal clock source
;                       Prescale = 1:64
;       Code Protect:   Disabled
;
;       Subroutines:    None
;       Parameters:     None
;
; **************************************************************

	PROCESSOR 16F84A

; Register Label Equates........................................        
        
TMR0    EQU     01             ; Counter/Timer Register
PORTB   EQU     06             ; Port B Data Register (LEDs)
INTCON  EQU     0B             ; Interrupt Control Register

T0IF    EQU     2              ; Timer Interrupt Flag

; **************************************************************

; Initialise Port B (Port A defaults to inputs).................
        
      	 MOVLW   b'00000000'     ; Set Port B Data Direction 
    	 TRIS    PORTB      

      	 MOVLW   b'00000101'     ; Set up Option register
      	 OPTION                  ; for internal timer/64

	 CLRF    PORTB           ; Clear Port B (LEDs Off)


;  Main output loop ............................................

next 	 CLRF    TMR0            ; clear timer register
      	 BCF     INTCON,T0IF     ; clear timeout flag 

check	 BTFSS   INTCON,T0IF     ; wait for next timeout
      	 GOTO    check           ; by polling timeout flag

	 INCF    PORTB           ; Increment LED Count
      	 GOTO    next            ; repeat forever...

      	 END                     ; Terminate source code


