; *********************************************************
;       CLS2.ASM	MPB 	Ver1.0
;	Status: Working OK in simulation 17-12-10 
; *********************************************************
;       Closed Loop DC Motor Speed Control using Pulse 
;	Width Modulation (software loop) to control speed 
;	and hardware timer to set reference time interval
;
;	Hardware:	MOT2 Proteus VSM simulation
;       MCU:		16F690, 1MHz internal clock
;       Inputs:         RC0-RC7 DIP switches
;                       RA2 index sensor (high pulse)
;       Outputs:        RA4,5 Motor forward, reverse       
;       
; Set Processor Options....................................

 	PROCESSOR 16F690	; Specify MCU for assembler
				; MCU configuration bits
	__CONFIG 00E4		; PWRT on, MCLR enabled 
				; Internal Clock (1MHz)
	INCLUDE "P16F690.INC"	; Standard register labels

; Register Label Equates...................................
        
Speed	EQU	020		; Counter Pre-load Value
Count	EQU	021		; Delay Counter
Flags	EQU	022		; User Flags

; Register Bit Label Equates ..............................
  
forwd   EQU    	4              	; Motor Forward = RA4
revrs	EQU	5	 	; Motor Reverse = RA5
sensor	EQU	2		; Shaft Opto-Sensor = RA2
slot	EQU	0		; Slot Found Flag
done	EQU	1		; Time Out Done Flag
timout	EQU	2		; Time Out Flag = TMR0,2

; Initialise, Port B defaults to input....................
        
	BANKSEL	ANSEL		; Select Bank 2
	CLRF	ANSEL		; All digital I/O
	CLRF	ANSELH		; ..initially
	BSF	ANSELH,2	; Analogue inputs
	BSF	ANSELH,3	; ..AN10, AN11

	BANKSEL	TRISA		; Select Bank 1
	MOVLW	B'11001111'	; RA4, RA5 output
	MOVWF	TRISA		; and load dirc. reg.
	MOVLW	B'10000111'	; Code for TMR0..
	MOVWF	OPTION_REG	; sets prescale 1:256
	MOVLW	B'01000111'	; Code to select..
	MOVWF	OSCCON		; internal clock = 1MHz


	BANKSEL PORTA		; Select Bank 0	
	CLRF	PORTA		; Motor off 
	MOVLW	d'100'		; Initial value for
        MOVWF  	Speed          	; ..speed
	CLRF	Flags		; and clear user flags
	GOTO	start		; Jump to main program 

; Subroutine reloads Timer0...............................

reltim	MOVF	PORTC,W		; Get input switches &..
	MOVWF	TMR0		; Load Timer with input
	BCF	INTCON,timout	; Reset 'TimeOut' Flag
	RETURN

; Subroutine checks for time out or slot...................

testem	BTFSS	INTCON,timout	; Time Out?
	GOTO	tessen		; NO: Skip Speed Increment
	BSF	Flags,done	; YES: Set Time Out Flag
	INCFSZ	Speed		; Test for maximum speed
	GOTO	reload		; NO: jump to timer reload
	DECF	Speed		; YES: Decrement to 255 
	GOTO	reload		; & jump to timer reload

tessen	BTFSC	PORTA,sensor	; Slot Present?
	GOTO	teslot		; YES: jump to test slot
	BCF	Flags,slot	; NO: Reset 'Slot' Flag
	GOTO	datcon		; & continue Count loop

teslot	BTFSC	Flags,slot	; 'Slot' Flag Set?
	GOTO	datcon		; YES: Skip speed decrement
	BTFSC	Flags,done	; NO: 'Done' Flag Set?
	GOTO	clrdone		; YES: Skip speed decrement
	DECFSZ	Speed		; NO: Test for min. speed
	GOTO	clrdone		; NO: continue loop
	INCF	Speed		; YES: increment back to 1

clrdone	BCF	Flags,done	; Clear 'Done' Flag
setslot	BSF	Flags,slot	; Set 'Slot' Flag
reload	CALL	reltim		; Reload timer
datcon	DECFSZ	Count		; Decrement & Test Count
	GOTO	testem		; Counter not zero yet
	RETURN			; End motor cycle if zero
	
; Drive loop outputs one cycle of PWM to motor .............

start	CALL	reltim		; reload timer to start

again	BSF	PORTA,forwd	; Motor ON
	MOVF	Speed,W		; Put ON delay value 
	MOVWF	Count		; into counter
	CALL	testem		; Modify speed

	BCF	PORTA,forwd	; Motor OFF
	MOVF	Speed,W		; Put ON delay value
	MOVWF	Count		; into counter
	COMF	Count		; and convert to OFF value
	BTFSC	STATUS,Z	; Test for zero count
	INCF	Count		; ..and avoid
	CALL	testem		; Modify speed
	GOTO	again		; next drive cycle

        END	; Terminate source code  ********************



