;**************************************************************
;	HOB1.ASM	MPB  Ver 1.0
;	Test program for LPC demo board
;	Pulse output to servo on RC5
;	
;	Internal clock = 4MHz
;	Variable frequency pulse output to servo
;	0.5 - 1.5 - 2.5 ms pulse, overall 18ms period
;
;	Status: working 21-12-10  
;
;**************************************************************

	PROCESSOR 16F690	; Specify MCU for assembler

	INCLUDE "P16F690.INC"	; MCU register lables

	Count	EQU	20	; Delay count
	Two	EQU	21	; Delay multiplier
	Ten	EQU	22	; Delay multiplier
	HiPer	EQU	23	; High period count
	LoPer	EQU	24	; Low period count
	ADres	EQU	25	; AD result store

; Initialise registers.........................................

	BANKSEL	ANSEL		; Select Bank 2
	CLRF	ANSEL		; Ports digital I/O
	CLRF	ANSELH		; Ports digital I/O
	BSF	ANSEL,0		; except AN0 Analogue input

	BANKSEL	TRISC		; Select Bank 1
	CLRF	TRISC		; Initialise Port C for output
	MOVLW	B'00000000'	; Analogue input setup code
	MOVWF	ADCON1		; Left justify result, ref=Vdd

	BANKSEL PORTC		; Select bank 0
	CLRF	PORTC		; Clear display outputs
	MOVLW	B'00000001'	; Analogue input setup code
	MOVWF	ADCON0		; f/8, RA0, done, enable  

; Start main loop.............................................
		
getpot	BSF	ADCON0,1	; Start ADC..
wait	BTFSC	ADCON0,1	; ..and wait for finish
	GOTO	wait
	MOVF	ADRESH,W	; Test result
	BTFSC	STATUS,Z	
	INCF	ADRESH		; Avoid zero
	MOVF	ADRESH,W	; store result 
	MOVWF	HiPer		; Store high time
	MOVWF	LoPer
	COMF	LoPer		; Calculate low time
	BTFSC	STATUS,Z	
	INCF	LoPer		; Avoid zero

	BSF	PORTC,5		; output high pulse
	MOVWF	Count
	CALL 	by12		; and wait for HiPer

	BCF	PORTC,5		; Output low
	MOVF	LoPer,W
	MOVWF	Count
	CALL 	by12		; and wait for LoPer

	MOVLW	D'10'		; Extra delay for 15ms
	MOVWF	Ten		; to make total cycle
by4	MOVLW	D'125'		; time = 18ms
	MOVWF	Count
	CALL	by12
	DECFSZ	Ten
	GOTO	by4

	GOTO	getpot		; Repeat main loop
	
; Wait for Count x 12us ......................................

by12	MOVLW	02		; Loop delay adjusted
	MOVWF	Two		; in simulation
by3	NOP
	DECFSZ	Two
	GOTO	by3
	DECFSZ	Count		
	GOTO	by12
	RETURN

	END	; Terminate assembler ***********************

