; *********************************************************
;       MOT1.ASM            M. Bates        14/6/99       
; *********************************************************
;
;       DC Motor Control using Pulse Width Modulation
;       Motor (RA0) starts with 50% MSR when enabled with 
;       RA4. Speed controlled with RA2, RA3.
;
;       Hardware:       Simple Motor Circuit   
;       Clock:          CR ~100kHz
;       Inputs:         Push Buttons (active low):    
;                       RA2 = Speed Up
;                       RA3 = Slow Down
;                       RA4 = Run
;       Outputs:        RA0 (active high) = Motor
;       
;       Chip Fuse Settings:
;       WDTimer:        Disable
;       PUTimer:        Enable
;       Interrupts:     Disable
;       Code Protect:   Disable
;
; *********************************************************

  PROCESSOR 16F84A

; Register Label Equates...................................        
        
PORTA   EQU     05              ; Port A   
Timer   EQU     0C              ; Delay Counter
Count   EQU     0D              ; Delay Count Pre-load

; Input Bit Label Equates .................................

motor	EQU     0               ; Motor Output = RA0
up	EQU     2               ; Speed Up Input = RA2 
down	EQU     3               ; Slow Down Input = RA3
run	EQU     4               ; Motor Enable Input = RA4

; *********************************************************

; Initialise ..............................................
        
       	MOVLW   b'11111110'     ; Port A bit direction code
       	TRIS    PORTA           ; Set the bit direction
	MOVWF	PORTA		; Initialise input bits
        MOVLW   080             ; Initial value
        MOVWF   Count           ; ...for delay

; Input Test ..............................................

start   BTFSC   PORTA,run       ; Test Run input
        GOTO    start           ; & wait if HIGH

        BTFSS   PORTA,up        ; Test Up input, if hi
        INCFSZ  Count           ; ...inc Count, test
        GOTO    test            ; and check down button
        DECF    Count           ; or dec Count again if 00

test    BTFSS   PORTA,down      ; Test Down input, if hi
        DECFSZ  Count           ; ...dec Count, test  
        GOTO    cycle           ; and do an output cycle
        INCF    Count           ; or inc Count again if 00

; Output High and Delay ...................................

cycle   BSF     PORTA,motor     ; Switch on motor 
        
        MOVF    Count,W         ; Get delay count
        MOVWF   Timer           ; Load timer register
again1  DECFSZ  Timer           ; Decrement timer register 
        GOTO    again1          ; & repeat until zero then


; Output Low and Delay ....................................

        BCF     PORTA,motor     ; Switch off motor
        
        MOVF    Count,W         ; Get delay count again
        MOVWF   Timer           ; Reload timer register
        COMF    Timer           ; Complement timer value
        INCF    Timer           ; Inc to avoid 00 value
again2  DECFSZ  Timer           ; Decrement timer register 
        GOTO    again2          ; & repeat until zero then

; Repeat Endlessly ........................................

        GOTO    start           ; Restart main loop 
        END                     ; Terminate source code




