;************************************************************
;	POS2.ASM	MPB 	Ver1.0
;	Test program for motor demo board MOT2
; 	Position control
;	Pot sets target position, move triggered by S1
;	Index pulse count in TMR0
;	
;	Working in simulation mode 16-2-11
;	DCM servo settings: 6V, 3R, 1mH, 1000rpm, mass=0.0003
;
;************************************************************

	PROCESSOR 16F690	; Specify MCU for assembler
				; MCU configuration bits
	__CONFIG 00E4		; PWRT on, MCLR enabled 
				; Internal Clock -
				; Default = 4MHz
	INCLUDE "P16F690.INC"	; Standard register labels

PotLas	EQU	020		; Pot start position
PotNow	EQU	021		; Pot target position
Count1	EQU	022		; Overrun counter
Count2	EQU	023		; Overrun counter
PotDif	EQU	024		; Pot change or overrun	
Timer1	EQU	025		; Delay timers
Timer2	EQU	026		; 
Timer3	EQU	027		;
OverCo	EQU	028		; Overcount holding

; Initialise registers.......................................

	CLRF	PORTA		; Motor off

	BANKSEL	ANSEL		; Select Bank 2
	CLRF	ANSEL		; All digital I/O
	CLRF	ANSELH		; ..initially
	BSF	ANSELH,2	; Analogue inputs
	BSF	ANSELH,3	; ..AN10, AN11

	BANKSEL	TRISA		; Select Bank 1
	MOVLW	B'11001111'	; RA4, RA5 output
	MOVWF	TRISA		; 
	MOVLW	B'01110000'	; A/D clock setup code
	MOVWF	ADCON1		; Internal clock

	BANKSEL ADCON0		; Select Bank 0
	MOVLW	B'00101001'	; Analogue setup code
	MOVWF	ADCON0		; Left justify, Vref=5V, 
				; RA10, done, enable A/D 
	MOVLW	D'127'		; Mid value
	MOVWF	PotLas		; ..into position regs
	MOVWF	PotNow

	CLRF	PORTA		; Switch off motor
	CLRF	PotDif		; Zero pot movement

; Main loop ................................................

start	BTFSC	PORTB,6		; Wait for S1
	GOTO	start

getpot	MOVF	PotNow,W	; Save previous pot input	
	MOVWF	PotLas

	BSF	ADCON0,1	; Start ADC..
finish	BTFSC	ADCON0,1	; ..and wait for finish
	GOTO	finish
	MOVF	ADRESH,W	; Store result high byte
	MOVWF	PotNow		; Current pot value

	BCF	STATUS,Z	; Clear zero flag
	BSF	STATUS,C	; Set carry flag
	MOVF	PotLas,W	; 
	SUBWF	PotNow,W	; W = PotNow - PotLas
	MOVWF	PotDif		;
	 
	BTFSC	STATUS,Z	; If PotDif = 0
	GOTO	start
	BTFSC	STATUS,C	; Pot moved negative?
	GOTO	forwrd
	COMF	PotDif		; Convert to positive
	GOTO	revers		; yes - reverse motor
	
forwrd	BCF	PORTA,5		; Reverse off
	BSF	PORTA,4		; Motor forward
	CALL	wait
	MOVF	PotDif,W	; Motor at target?
	BTFSC	STATUS,Z	;
	GOTO	start		; Yes - start again

revers	BCF	PORTA,4		; Forward off
	BSF	PORTA,5		; Motor reverse
	CALL	wait
	MOVF	PotDif,W	; Motor at target?
	BTFSC	STATUS,Z	;
	GOTO	start		; Yes - start again

	GOTO	forwrd		; No - keep adjusting

; Subroutine to stop motor and to correct overrun ........

wait	CLRF	TMR0		; Count motor pulses
check	MOVF	TMR0,W		
	SUBWF	PotDif,W
	BTFSS	STATUS,Z	; until target reached
	GOTO	check
	CLRF	PORTA		; Motor off

	CLRF	TMR0		; Reset pulse count
stop	MOVF	TMR0,W
	MOVWF	Count1		; Store pulse count
	CALL	long		; Wait a bit

	MOVF	TMR0,W		; Store count again
	MOVWF	Count2
	MOVWF	OverCo
	SUBWF	Count1		; Check if changed
	BTFSS	STATUS,Z
	GOTO	stop		; ..until unchanged
	CALL	long		; Wait a bit
	
	MOVF	OverCo,W	; store overcount
	MOVWF	PotDif		
	RETURN	

; Long delay............................................

long	MOVLW	D'10'
	MOVWF	Timer1		; 1s
loop1	MOVLW	D'100'
	MOVWF	Timer2		; 100ms
loop2	MOVLW	D'249'
	MOVWF	Timer3		; 1ms
loop3	NOP			
	DECFSZ	Timer3		
	GOTO	loop3
	DECFSZ	Timer2		
	GOTO	loop2
	DECFSZ	Timer1
	GOTO	loop1
	RETURN

	END	; Terminate assembler...................

